# -*- coding: utf-8 -*-
from datetime import datetime
from HTMLParser import HTMLParseError

from django.contrib import admin
from django.contrib import messages
from django.forms.widgets import PasswordInput
from django.utils.translation import ugettext_lazy as _
from django import forms
from django.contrib.admin.options import IncorrectLookupParameters
from django.core.exceptions import ValidationError

from jumbo.admin.options import ExtendibleModelAdmin, SettingsAdmin
from jumbo.forms.fields import JumboDateField
from jumbo.core.urlresolvers import reverse

from mailer.models import EmailObject, Attachment, Settings, LinkStatus
from mailer.models import MailingStatus, BounceServer
from mailer.models import SMTPServer, Link
from mailer.sender import Mailer

##############################
# EMAIL                      #
##############################
class AttachmentAdminInline(admin.TabularInline):
    model = Attachment
    extra = 1
    fieldsets = ((None, {'fields': (('title', 'file_attachment'))}),)

class EmailObjectSearchForm(forms.ModelForm):
    class Meta:
        model = EmailObject
        display_fields = (
            'status',
            ('date_create__gte', 'date_create__lte',),
            'email_address__icontains',
            'subject__icontains',
            'opened_mail'
        )
    STATUS = EmailObject.STATUS_CHOICES
    STATUS.insert(0, ('', _('---')))
    OPENED = (
        ('', _('---')),
        (1, _('Yes')),
        (0, _('No'))
    )

    def __init__(self, *args, **kws):
        super(EmailObjectSearchForm, self).__init__(*args, **kws)
        self.fields['status'] = forms.ChoiceField(label=_("Status"), required=False, choices=self.STATUS)
        self.fields['date_create__gte'] = JumboDateField(label=_("Date create from"), required=False)
        self.fields['date_create__lte'] = JumboDateField(label=_("Date create to"), required=False)
        self.fields['email_address__icontains'] = forms.CharField(label=_("Email icontains"), required=False)
        self.fields['subject__icontains'] = forms.CharField(label=_("Subject icontains"), required=False)
        self.fields['opened_mail'] = forms.ChoiceField(label=_("Opened"), required=False, choices=self.OPENED)

    def change_search_filter(self, lookup_params, qs):
        if lookup_params.has_key('date_create__lte'):
            date = lookup_params['date_create__lte']
            try:
                fixed_datetime = datetime(date.year, date.month, date.day, 23, 59, 59)
                qs = qs.filter(date_create__lte = fixed_datetime)
            except Exception, e:
                raise IncorrectLookupParameters
            del lookup_params['date_create__lte']

        if lookup_params.has_key('opened_mail'):
            opened_mail = lookup_params['opened_mail']
            if opened_mail == "1":
                qs = qs.filter(
                    mailing_status__status__in=(
                        MailingStatus.OPENED,
                        MailingStatus.OPENED_ON_SITE,
                        MailingStatus.LINK_OPENED
                    )
                ).distinct()
            if opened_mail == "0":
                qs = qs.exclude(
                    mailing_status__status__in=(
                        MailingStatus.OPENED,
                        MailingStatus.OPENED_ON_SITE,
                        MailingStatus.LINK_OPENED
                    )
                ).distinct()
            del lookup_params['opened_mail']
        return lookup_params, qs

class EmailObjectAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("mailer/css/admin_mailer.css",)}

    date_hierarchy = 'date_create'
    search_form = EmailObjectSearchForm
    list_display = (
        'status', 'date_create', 'email_address', 'subject', 'smtp_server', 'bounce_server',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {'fields': (
            'status',
            'email_address',
            'subject',
            'content',
        )}),
        (_('Configuration'), {
            'fields': (
                ('smtp_server', 'bounce_server',),
                ('header_sender', 'header_reply',),
                'slug',
            ),
            'classes': ('collapse',)
        }),
    )
    prepopulated_fields = {'slug': ('subject',)}
    inlines = (AttachmentAdminInline,)
    actions = ['send_mail_test', 'make_ready_to_send', 'make_cancel_sending']

    def get_actions(self, request):
        actions = super(EmailObjectAdmin, self).get_actions(request)
        try:
            del actions['set_active']
            del actions['set_disactive']
            del actions['set_as_completed']
            del actions['set_as_to_complete']
            if not request.user.has_perm('emailobject.can_change_status'):
                del actions['make_ready_to_send']
                del actions['make_cancel_sending']
        except:
            pass
        return actions

#    def queryset(self, request):
#        queryset = super(EmailAdmin, self).queryset(request)
#        if not request.user.is_superuser and USE_WORKGROUPS:
#            newsletters_pk = request_workgroups_newsletters_pk(request)
#            queryset = queryset.filter(pk__in=newsletters_pk)
#        return queryset

#    def formfield_for_foreignkey(self, db_field, request, **kwargs):
#        if db_field.name == 'mailing_list' and \
#               not request.user.is_superuser and USE_WORKGROUPS:
#            mailinglists_pk = request_workgroups_mailinglists_pk(request)
#            kwargs['queryset'] = MailingList.objects.filter(pk__in=mailinglists_pk)
#            return db_field.formfield(**kwargs)
#        return super(NewsletterAdmin, self).formfield_for_foreignkey(
#            db_field, request, **kwargs)

#    def formfield_for_choice_field(self, db_field, request, **kwargs):
#        if db_field.name == 'status' and \
#               not request.user.has_perm('newsletter.can_change_status'):
#            kwargs['choices'] = ((Newsletter.DRAFT, _('Default')),)
#            return db_field.formfield(**kwargs)
#        return super(NewsletterAdmin, self).formfield_for_choice_field(
#            db_field, request, **kwargs)

#    def formfield_for_manytomany(self, db_field, request, **kwargs):
#        if db_field.name == 'test_contacts':
#            queryset = Contact.objects.filter(tester=True)
#            if not request.user.is_superuser and USE_WORKGROUPS:
#                contacts_pk = request_workgroups_contacts_pk(request)
#                queryset = queryset.filter(pk__in=contacts_pk)
#            kwargs['queryset'] = queryset
#        return super(NewsletterAdmin, self).formfield_for_manytomany(
#            db_field, request, **kwargs)

#    def save_model(self, request, newsletter, form, change):
#        workgroups = []
#        if not newsletter.pk and not request.user.is_superuser \
#               and USE_WORKGROUPS:
#            workgroups = request_workgroups(request)
#
#        if newsletter.content.startswith('http://'):
#            if CAN_USE_PREMAILER:
#                try:
#                    premailer = Premailer(newsletter.content.strip())
#                    newsletter.content = premailer.transform()
#                except PremailerError:
#                    self.message_user(request, _('Unable to download HTML, due to errors within.'))
#            else:
#                self.message_user(request, _('Please install lxml for parsing an URL.'))
#        if not request.user.has_perm('newsletter.can_change_status'):
#            newsletter.status = form.initial.get('status', Newsletter.DRAFT)
#
#        newsletter.save()
#
#        for workgroup in workgroups:
#            workgroup.newsletters.add(newsletter)

    def send_mail_test(self, request, queryset):
        """Send newsletter in test"""
        for email in queryset:
            if email.status == EmailObject.TO_SEND_IN_TEST:
                mailer = Mailer(email, test=True, verbose=2)
                try:
                    mailer.send()
                except HTMLParseError:
                    messages.error(request, _('Unable send mail, due to errors within HTML.'))
                    continue
                self.message_user(request, _('%s succesfully sent.') % email)
            else:
                messages.error(request, _('%s is not email test.') % email)

    send_mail_test.short_description = _('Send test email')

    def make_ready_to_send(self, request, queryset):
        """Make emails ready to send"""
        queryset = queryset.filter(status=EmailObject.DRAFT)
        for email in queryset:
            email.status = EmailObject.TO_SEND
            email.save()
        self.message_user(request, _('%s emails are ready to send') % queryset.count())
    make_ready_to_send.short_description = _('Make ready to send')

    def make_cancel_sending(self, request, queryset):
        """Cancel the sending of newsletters"""
        queryset = queryset.filter(
            Q(status=EmailObject.TO_SEND)
        )
        for email in queryset:
            email.status = EmailObject.CANCELED
            email.save()
        self.message_user(request, _('%s emails are cancelled') % queryset.count())
    make_cancel_sending.short_description = _('Cancel the sending')


##############################
# BOUNCER                    #
##############################
class BounceServerAdminForm(forms.ModelForm):
    def __init__(self, *args, **kws):
        super(BounceServerAdminForm, self).__init__(*args, **kws)
        self.fields['password'] = forms.CharField(widget=PasswordInput(), required = False)

    class Meta:
        model = BounceServer

class BounceServerAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("mailer/css/admin_mailer.css",)}

    form = BounceServerAdminForm
    list_display = (
        'name', 'host', 'port', 'user', 'tls',
        'get_edit_icon', 'get_delete_icon'
    )
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {'fields': ('name', )}),
        (_('Configuration'), {'fields': (
            'host', 'port', 'user', 'password', 'tls',
        )}),
    )

    actions = ['check_connections']
    def get_actions(self, request):
        actions = super(BounceServerAdmin, self).get_actions(request)
        try:
            del actions['set_active']
            del actions['set_disactive']
            del actions['set_as_completed']
            del actions['set_as_to_complete']
        except:
            pass
        return actions

    def check_connections(self, request, queryset):
        """Check the Bouncer connection"""
        message = '%s connection %s'
        for server in queryset:
            try:
                bouncer = server.connect()
                if bouncer:
                    status = 'OK'
                    bouncer.quit()
                else:
                    status = 'KO'
            except:
                status = 'KO'
            self.message_user(request, message % (server.__unicode__(), status))
    check_connections.short_description = _('Check connection')

##############################
# SMTP                       #
##############################
class SMTPServerAdminForm(forms.ModelForm):
    def __init__(self, *args, **kws):
        super(SMTPServerAdminForm, self).__init__(*args, **kws)
        self.fields['password'] = forms.CharField(widget=PasswordInput(), required = False)

    def clean_headers(self):
        """Check if the headers are well formated"""
        for line in self.cleaned_data['headers'].splitlines():
            elems = line.split(':')
            if len(elems) < 2:
                raise ValidationError(_('Invalid syntax, do not forget the ":".'))
            if len(elems) > 2:
                raise ValidationError(_('Invalid syntax, several assignments by line.'))

        return self.cleaned_data['headers']

    class Meta:
        model = SMTPServer

class SMTPServerAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("mailer/css/admin_mailer.css",)}

    form = SMTPServerAdminForm
    list_display = (
        'name', 'host', 'port', 'user', 'tls', 'get_credits',
        'get_edit_icon', 'get_delete_icon'
    )
    #list_filter = ('tls',)
    #search_fields = ('name', 'host', 'user')
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {'fields': ('name', )}),
        (_('Configuration'), {'fields': (
            'host', 'port', 'user', 'password', 'tls', 'mails_hour',
        )}),
        (_('Miscellaneous'), {
            'fields': ('headers',),
            'classes': ('collapse', )
        }),
    )
    actions = ['check_connections']
    def get_actions(self, request):
        actions = super(SMTPServerAdmin, self).get_actions(request)
        try:
            del actions['set_active']
            del actions['set_disactive']
            del actions['set_as_completed']
            del actions['set_as_to_complete']
        except:
            pass
        return actions

#    inlines = [CreditAdminInline,]

    def check_connections(self, request, queryset):
        """Check the SMTP connection"""
        message = '%s connection %s'
        for server in queryset:
            try:
                smtp = server.connect()
                if smtp:
                    status = 'OK'
                    smtp.quit()
                else:
                    status = 'KO'
            except:
                status = 'KO'
            self.message_user(request, message % (server.__unicode__(), status))
    check_connections.short_description = _('Check connection')

#    def get_urls(self):
#        from django.conf.urls.defaults import patterns, url
#        urls = super(SMTPServerAdmin, self).get_urls()
#        my_urls = patterns('',
#            url(
#                r'^(?P<smtp_id>\d+)/set_credits/$',
#                self.admin_site.admin_view(newsletter_views.set_credits),
#                name='newsletter_smtpserver_set_credits'
#            ),
#        )
#        return my_urls + urls

#    def set_credits(self, obj):
#        return """
#            <a class="iframe" href="%(url_reverse)s?_popup=1&nobutton=1">
#                <img src='%(media_url)sadmin/images/comment.png' title=\"Set Credits\" style=\"vertical-align:middle;\" />
#            </a>
#        """ % {
#            'url_reverse': reverse('admin:newsletter_smtpserver_set_credits', args=str(obj.id)),
#            'media_url': settings.MEDIA_URL,
#        }
#    set_credits.allow_tags=True


##############################
# LINK                       #
##############################
class LinkSearchForm(forms.ModelForm):
    class Meta:
        model = Link
        display_fields = (
            ('date_create__gte', 'date_create__lte',),
            ('email_object__email_address__icontains', 'email_object__subject__icontains'),
            'title__icontains',
            'url__icontains'
        )
    def __init__(self, *args, **kws):
        super(LinkSearchForm, self).__init__(*args, **kws)
        self.fields['date_create__gte'] = JumboDateField(label=_("Date create from"), required=False)
        self.fields['date_create__lte'] = JumboDateField(label=_("Date create to"), required=False)
        self.fields['email_object__email_address__icontains'] = forms.CharField(label=_("Email icontains"), required=False)
        self.fields['email_object__subject__icontains'] = forms.CharField(label=_("Subject icontains"), required=False)
        self.fields['title__icontains'] = forms.CharField(label=_("Title icontains"), required=False)
        self.fields['url__icontains'] = forms.CharField(label=_("URL icontains"), required=False)

    def change_search_filter(self, lookup_params, qs):
        if lookup_params.has_key('date_create__lte'):
            date = lookup_params['date_create__lte']
            try:
                fixed_datetime = datetime(date.year, date.month, date.day, 23, 59, 59)
                qs = qs.filter(date_create__lte = fixed_datetime)
            except Exception, e:
                raise IncorrectLookupParameters
            del lookup_params['date_create__lte']
        return lookup_params, qs

class LinkAdmin(ExtendibleModelAdmin):
    date_hierarchy = 'date_create'
    search_form = LinkSearchForm
    list_display = (
        'date_create', 'get_email', 'title', 'url',
        'get_edit_icon', 'get_delete_icon'
    )
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {'fields': (
            'email_object',
            'title',
            'url',
        )}),
    )

    def get_actions(self, request):
        actions = super(LinkAdmin, self).get_actions(request)
        try:
            del actions['set_active']
            del actions['set_disactive']
            del actions['set_as_completed']
            del actions['set_as_to_complete']
        except:
            pass
        return actions

    def get_email(self, result):
        return "%s: %s" % (
            result.email_object.email_address,
            result.email_object.subject
            )
    get_email.short_description = _("Email")

##############################
# MAILING STATUS             #
##############################
class MailingStatusSearchForm(forms.ModelForm):
    class Meta:
        model = MailingStatus
        display_fields = (
            ('status', 'rfc_status__icontains', 'diagnostic_code__icontains'),
            ('date_create__gte', 'date_create__lte',),
            ('email_object__email_address__icontains', 'email_object__subject__icontains',),
            'opened_email'
        )

    STATUS = MailingStatus.STATUS_CHOICES
    STATUS.insert(0, ('', _('---')))

    def __init__(self, *args, **kws):
        super(MailingStatusSearchForm, self).__init__(*args, **kws)
        self.fields['status'] = forms.ChoiceField(label=_("Status"), choices=self.STATUS, required=False)
        self.fields['date_create__gte'] = JumboDateField(label=_("Sent Date from"), required=False)
        self.fields['date_create__lte'] = JumboDateField(label=_("Sent Date to"), required=False)
        self.fields['email_object__email_address__icontains'] = forms.CharField(label=_("Email icontains"), required=False)
        self.fields['email_object__subject__icontains'] = forms.CharField(label=_("Subject icontains"), required=False)
        self.fields['rfc_status__icontains'] = forms.CharField(label=_("RFC Status icontains"), required=False)
        self.fields['diagnostic_code__icontains'] = forms.CharField(label=_("Diagnostic Code icontains"), required=False)
        self.fields['opened_email'] = forms.BooleanField(label=_("Opened"), required=False)

    def change_search_filter(self, lookup_params, qs):
        # data invio
        if lookup_params.has_key('date_create__lte'):
            date = lookup_params['date_create__lte']
            try:
                fixed_datetime = datetime(date.year, date.month, date.day, 23, 59, 59)
                qs = qs.filter(date_create__lte = fixed_datetime)
            except Exception, e:
                raise IncorrectLookupParameters
            del lookup_params['date_create__lte']

        # opened
        if lookup_params.has_key('opened_email'):
            opened = lookup_params['opened_email']
            if opened == "on":
                qs = qs.filter(status__in=(
                    MailingStatus.OPENED,
                    MailingStatus.OPENED_ON_SITE,
                    MailingStatus.LINK_OPENED
                ))
            del lookup_params['opened_email']

        return lookup_params, qs
        
class MailingStatusAdmin(ExtendibleModelAdmin):
    date_hierarchy = 'date_create'
    search_form = MailingStatusSearchForm
    list_display = (
        'get_status', 'date_create', 'get_email', 'get_link',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {'fields': (
            'status',
            'email_object',
            'rfc_status',
            'diagnostic_code'
        )}),
    )

    def get_actions(self, request):
        actions = super(MailingStatusAdmin, self).get_actions(request)
        try:
            del actions['set_active']
            del actions['set_disactive']
            del actions['set_as_completed']
            del actions['set_as_to_complete']
        except:
            pass
        return actions

    def get_email(self, result):
        return "%s: %s" % (
            result.email_object.email_address,
            result.email_object.subject
            )
    get_email.short_description = _("Email")

    def get_link(self, result):
        field = ""
        if result.status_link:
            field = """
                <a href="%s" title="%s" target="_blank">%s</a>
            """ % (
                result.status_link.link.url,
                result.status_link.link.title,
                result.status_link.link.title
            )
        return field
    get_link.short_description = _("Link")
    get_link.allow_tags = True

    def get_status(self, result):
        if result.rfc_status:
            field = """
                <span title="%s: %s">%s</span>
            """ % (
                result.rfc_status,
                result.diagnostic_code or "",
                result.get_status_display()
            )
        else:
            field = """
                <span title="%s">%s</span>
            """ % (
                result.diagnostic_code or "",
                result.get_status_display()
            )
        return field        
    get_status.short_description = _("Status")
    get_status.allow_tags = True

##############################
# LINK STATUS                #
##############################
class LinkStatusSearchForm(forms.ModelForm):
    class Meta:
        model = LinkStatus
        display_fields = (
            ('date_create__gte', 'date_create__lte',),
            ('mailing_status__email_object__email_address__icontains', 'mailing_status__email_object__subject__icontains',),
            ('link__title__icontains', 'link__url__icontains'),
        )

    def __init__(self, *args, **kws):
        super(LinkStatusSearchForm, self).__init__(*args, **kws)
        self.fields['date_create__gte'] = JumboDateField(label=_("Sent Date from"), required=False)
        self.fields['date_create__lte'] = JumboDateField(label=_("Sent Date to"), required=False)
        self.fields['mailing_status__email_object__email_address__icontains'] = forms.CharField(label=_("Email icontains"), required=False)
        self.fields['mailing_status__email_object__subject__icontains'] = forms.CharField(label=_("Subject icontains"), required=False)
        self.fields['link__title__icontains'] = forms.CharField(label=_("Link Title icontains"), required=False)
        self.fields['link__url__icontains'] = forms.CharField(label=_("Link URL icontains"), required=False)

    def change_search_filter(self, lookup_params, qs):
        # data invio
        if lookup_params.has_key('date_create__lte'):
            date = lookup_params['date_create__lte']
            try:
                fixed_datetime = datetime(date.year, date.month, date.day, 23, 59, 59)
                qs = qs.filter(date_create__lte = fixed_datetime)
            except Exception, e:
                raise IncorrectLookupParameters
            del lookup_params['date_create__lte']
        return lookup_params, qs

class LinkStatusAdmin(ExtendibleModelAdmin):
    date_hierarchy = 'date_create'
    search_form = LinkStatusSearchForm
    list_display = (
        'date_create', 'get_email', 'get_link',
        'get_edit_icon', 'get_delete_icon'
    )
    fieldsets = (
        (_('logs admin'), {
            'fields':('date_create', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {'fields': (
            'mailing_status',
            'link',
        )}),
    )

    def get_actions(self, request):
        actions = super(LinkStatusAdmin, self).get_actions(request)
        try:
            del actions['set_active']
            del actions['set_disactive']
            del actions['set_as_completed']
            del actions['set_as_to_complete']
        except:
            pass
        return actions

    def get_email(self, result):
        return "%s: %s" % (
            result.mailing_status.email_object.email_address,
            result.mailing_status.email_object.subject
        )
    get_email.short_description = _("Email")

    def get_link(self, result):
        return "%s: %s" % (
            result.link.title,
            result.link.url
        )
    get_link.short_description = _("Link")

admin.site.register(Settings)
admin.site.register(EmailObject, EmailObjectAdmin)
admin.site.register(SMTPServer, SMTPServerAdmin)
admin.site.register(BounceServer, BounceServerAdmin)
admin.site.register(LinkStatus, LinkStatusAdmin)
admin.site.register(Link, LinkAdmin)
admin.site.register(MailingStatus, MailingStatusAdmin)
