# -*- coding: utf-8 -*-
from datetime import datetime, time
import poplib
from smtplib import SMTP

from django.db import models
from django.utils.encoding import smart_str, force_unicode
from django.utils.translation import ugettext as _

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel, HTMLField
from jumbo.db.models import Settings as JSettings
from jumbo.utils.functions import get_settings
from mailer.settings import EMAIL_BASE_PATH

class BounceServer(UserModel, DateModel):
    """Configuration of Bounce server"""
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255
    )
    host = models.CharField(
        verbose_name=_('server host'),
        max_length=255
    )
    user = models.CharField(
        verbose_name=_('server user'),
        max_length=128, blank=True,
        help_text=_('Leave it empty if the host is public.')
    )
    password = models.CharField(
        verbose_name=_('server password'),
        max_length=128, blank=True,
        help_text=_('Leave it empty if the host is public.')
    )
    port = models.IntegerField(
        verbose_name=_('server port'),
        default=110
    )
    tls = models.BooleanField(
        verbose_name=_('server use TLS')
    )

    def __unicode__(self):
        return '%s (%s)' % (self.name, self.host)

    class Meta:
        verbose_name = _('Bounce setting')
        verbose_name_plural = _('Bounce settings')

    def connect(self):
        """Connect the Bounce Server"""
        bouncer = poplib.POP3(self.host, self.port)
        bouncer.user(self.user)
        bouncer.pass_(self.password)
        return bouncer

class SMTPServer(UserModel, DateModel):
    """Configuration of a SMTP server"""
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255
    )
    host = models.CharField(
        verbose_name=_('server host'),
        max_length=255
    )
    user = models.CharField(
        verbose_name=_('server user'),
        max_length=128, blank=True,
        help_text=_('Leave it empty if the host is public.')
    )
    password = models.CharField(
        verbose_name=_('server password'),
        max_length=128, blank=True,
        help_text=_('Leave it empty if the host is public.')
    )
    port = models.IntegerField(
        verbose_name=_('server port'),
        default=25
    )
    tls = models.BooleanField(
        verbose_name=_('server use TLS')
    )
    headers = models.TextField(
        verbose_name=_('custom headers'),
        blank=True,
        help_text=_(
            'key1: value1 key2: value2, splitted by return line.\n'\
            'Useful for passing some tracking headers if your provider allows it.'
        )
    )
    mails_hour = models.IntegerField(
        verbose_name=_('mails per hour'),
        default=0
    )

    def connect(self):
        """Connect the SMTP Server"""
        smtp = SMTP(smart_str(self.host), int(self.port))
        smtp.ehlo_or_helo_if_needed()
        if self.tls:
            smtp.starttls()
            smtp.ehlo_or_helo_if_needed()

        if self.user or self.password:
            smtp.login(smart_str(self.user), smart_str(self.password))
        return smtp

    def get_credits(self):
        """Return how many mails the server can send"""
        now = datetime.today()
        last_hour = datetime(now.year, now.month, now.day, now.hour, 0, 0, 0)

        sent_curr_hour = MailingStatus.objects.filter(
            models.Q(status=MailingStatus.SENT) |
            models.Q(status=MailingStatus.SENT_TEST),
            email_object__smtp_server=self,
            date_create__gte=last_hour
        ).count()

        time_now = time(now.hour, now.minute)

        # Cerco se ci sono delle eccezioni ai credits
        exception_credits = self.credits.filter(
            dow_from__lte=now.weekday(),
            hour_from__lte=time_now,
            dow_to__gte=now.weekday(),
            hour_to__gte=time_now
        )

        if exception_credits:
            mails_hour = exception_credits[0].mails_hour
        else:
            mails_hour = self.mails_hour

        MAILER_HARD_LIMIT = get_settings("mailer.settings", "MAILER_HARD_LIMIT")
        # se mails_hour = 0 entra nel if
        if not mails_hour or mails_hour > MAILER_HARD_LIMIT:
            return MAILER_HARD_LIMIT

        return mails_hour - sent_curr_hour

    @property
    def custom_headers(self):
        if self.headers:
            headers = {}
            for header in self.headers.splitlines():
                if header:
                    key, value = header.split(':')
                    headers[key.strip()] = value.strip()
            return headers
        return {}

    def __unicode__(self):
        return '%s (%s)' % (self.name, self.host)

    class Meta:
        verbose_name = _('SMTP setting')
        verbose_name_plural = _('SMTP settings')

class Credit(UserModel, DateModel):
    """How many mails server can send in a particular day at particular time"""
    DOW_CHOICES = (
        (0, _('Monday')),
        (1, _('Tuesday')),
        (2, _('Wendesday')),
        (3, _('Thursday')),
        (4, _('Friday')),
        (5, _('Saturday')),
        (6, _('Sunday')),
    )
    HOUR_CHOICES = []
    for i in range(24):
        HOUR_CHOICES.append((time(i), '%s:00' % i))

    smtp_server = models.ForeignKey(
        SMTPServer,
        verbose_name=_("smtp server"),
        related_name='credits'
    )
    dow_from = models.IntegerField(
        choices=DOW_CHOICES,
        max_length=1
    )
    hour_from = models.TimeField(
        choices=HOUR_CHOICES
    )
    dow_to = models.IntegerField(
        choices=DOW_CHOICES,
        max_length=1
    )
    hour_to = models.TimeField(
        choices=HOUR_CHOICES
    )
    mails_hour = models.IntegerField(
        verbose_name=_('mails per hour'),
        default=0
    )

    def __unicode__(self):
        return u"%s" % self.mails_hour

    class Meta:
        verbose_name = _('Credit')
        verbose_name_plural = _('Credits')
        unique_together = ("smtp_server", "dow_from", "hour_from", "dow_to", "hour_to")
        ordering = ("smtp_server", "dow_from", "hour_from")

class EmailObject(UserModel, DateModel):
    """Email to be sended"""
    """Status of the mailing"""

    TO_SEND = 0
    TO_SEND_IN_TEST = 4
    DRAFT = 1
    SENT = 2
    SENT_TEST = 5
    CANCELED = 3

    STATUS_CHOICES = [
        (TO_SEND, _('to send')),
        (TO_SEND_IN_TEST, _('to send in test')),
        (DRAFT, _('draft')),
        (SENT, _('sent')),
        (SENT_TEST, _('sent in test')),
        (CANCELED, _('canceled')),
    ]

    status = models.IntegerField(
        verbose_name=_('status'),
        choices=STATUS_CHOICES
    )
    email_address = models.EmailField(
        verbose_name=_('email'),
    )
    subject = models.CharField(
        verbose_name=_('subject'),
        max_length=255,
    )
    content = HTMLField(
        verbose_name=_('content'),
        help_text=_('write HTML code or paste an URL.'),
    )
    smtp_server = models.ForeignKey(
        SMTPServer,
        verbose_name=_('smtp server'),
        related_name='emails',
    )
    bounce_server = models.ForeignKey(
        BounceServer,
        verbose_name=_('bouncer server'),
        related_name='emails',
    )
    header_sender = models.CharField(
        verbose_name=_('sender'),
        max_length=255,
        default=get_settings("mailer.settings", "DEFAULT_HEADER_SENDER"),
    )
    header_reply = models.CharField(
        verbose_name=_('reply to'),
        max_length=255,
        default=get_settings("mailer.settings", "DEFAULT_HEADER_REPLY"),
    )
    slug = models.SlugField(
        help_text=_('Used for displaying the mail'),
        unique=True,
    )

    @models.permalink
    def get_absolute_url(self):
        return 'mailer_email_preview', (self.slug,)

    @models.permalink
    def get_statistics_url(self):
        return 'mailer_email_statistics', (self.slug,)

    def __unicode__(self):
        return self.subject

    def set_status(self, mailing_status):
        """
        set status of email after sending
        """
        if mailing_status == MailingStatus.TO_SEND:
            self.status = EmailObject.TO_SEND
        elif mailing_status == MailingStatus.TO_SEND_IN_TEST:
            self.status = EmailObject.TO_SEND_IN_TEST
        elif mailing_status == MailingStatus.SENT:
            self.status = EmailObject.SENT
        elif mailing_status == MailingStatus.SENT_TEST:
            self.status = EmailObject.SENT_TEST
        elif mailing_status.status == MailingStatus.ERROR:
            self.status = EmailObject.SENT
        elif mailing_status.status == MailingStatus.INVALID:
            self.status = EmailObject.SENT
        self.save()

    class Meta:
        ordering = ('-date_create',)
        verbose_name = _('email')
        verbose_name_plural = _('emails')

class Attachment(UserModel, DateModel):
    def get_attachment_storage_path(self, filename):
        filename = force_unicode(filename)
        return '/'.join([EMAIL_BASE_PATH, filename])

    """Attachment file in an mail"""
    email_object = models.ForeignKey(
        EmailObject,
        verbose_name=_('email'),
        related_name="attachments"
    )
    title = models.CharField(
        verbose_name=_('title'),
        max_length=255
    )
    file_attachment = models.FileField(
        verbose_name=_('file to attach'),
        max_length=255,
        upload_to=get_attachment_storage_path
    )

    class Meta:
        verbose_name = _('attachment')
        verbose_name_plural = _('attachments')

    def __unicode__(self):
        return self.title

    def get_absolute_url(self):
        return self.file_attachment.url

class MailingStatus(UserModel, DateModel):
    """Status of the mailing"""
    TO_SEND = 0
    SENT_TEST = 1
    SENT = 2
    ERROR = 3
    INVALID = 4
    OPENED = 5
    OPENED_ON_SITE = 6
    LINK_OPENED = 7
    BOUNCED = 9
    TO_SEND_IN_TEST = 10

    STATUS_CHOICES = [
        (TO_SEND, _('to send')),
        (TO_SEND_IN_TEST, ('to send in test')),
        (SENT_TEST, _('sent in test')),
        (SENT, _('sent')),
        (ERROR, _('error')),
        (INVALID, _('invalid email')),
        (OPENED, _('opened')),
        (OPENED_ON_SITE, _('opened on site')),
        (LINK_OPENED, _('link opened')),
        (BOUNCED, _('bounced email')),
    ]

    status = models.IntegerField(
        verbose_name=_('status'),
        choices=STATUS_CHOICES
    )
    email_object = models.ForeignKey(
        EmailObject,
        verbose_name=_('email'),
        related_name="mailing_status"
    )
    rfc_status = models.CharField(
        max_length=10,
        blank=True, null=True
    )
    diagnostic_code = models.TextField(
        blank=True, null=True
    )

    def __unicode__(self):
        return '%s : %s' % (
            self.email_object,
            self.get_status_display()
        )

    class Meta:
        ordering = ('-date_create',)
        verbose_name = _('mailing status')
        verbose_name_plural = _('mailing status')

class Link(UserModel, DateModel):
    email_object = models.ForeignKey(
        EmailObject,
        verbose_name=_('email'),
        related_name="links"
    )
    title = models.CharField(
        verbose_name=_('title'),
        max_length=255
    )
    url = models.CharField(
        verbose_name=_('url'),
        max_length=255
    )

    def get_absolute_url(self):
        return self.url

    def __unicode__(self):
        return self.title

    class Meta:
        ordering = ('-date_create',)
        verbose_name = _('link')
        verbose_name_plural = _('links')

class LinkStatus(DateModel):
    mailing_status = models.OneToOneField(
        MailingStatus,
        verbose_name=_('mailing status'),
        related_name ='status_link'
    )
    link = models.ForeignKey(
        Link,
        verbose_name=_('link'),
        related_name = 'status_links',
    )

    class Meta:
        ordering = ('-date_create',)
        verbose_name = _('link status')
        verbose_name_plural = _('link status')

class Settings(JSettings):
    pass