"""Mailer for send email"""
import  re
import mimetypes
from random import sample
from StringIO import StringIO
from smtplib import SMTPRecipientsRefused

try:
    from email.mime.multipart import MIMEMultipart
    from email.mime.text import MIMEText
    from email.mime.Encoders import encode_base64
    from email.mime.MIMEAudio import MIMEAudio
    from email.mime.MIMEBase import MIMEBase
    from email.mime.MIMEImage import MIMEImage
except ImportError:  # Python 2.4 compatibility
    from email.MIMEMultipart import MIMEMultipart
    from email.MIMEText import MIMEText
    from email.Encoders import encode_base64
    from email.MIMEAudio import MIMEAudio
    from email.MIMEBase import MIMEBase
    from email.MIMEImage import MIMEImage
from email import message_from_file
from html2text import html2text as html2text_orig
from django.contrib.sites.models import Site
from django.template import Context, Template
from django.template.loader import render_to_string
from django.utils.encoding import smart_str
from django.utils.encoding import smart_unicode
from django.db import transaction

from mailer.models import MailingStatus, EmailObject
from mailer.utils.tokens import tokenize
from mailer.utils.mail_utils import track_links
from mailer.utils.mail_utils import body_insertion
from mailer.settings import TRACKING_LINKS
from mailer.settings import TRACKING_LINK_SITE
from mailer.settings import TRACKING_IMAGE
from mailer.settings import UNIQUE_KEY_LENGTH
from mailer.settings import UNIQUE_KEY_CHAR_SET


LINK_RE = re.compile(r"https?://([^ \n]+\n)+[^ \n]+", re.MULTILINE)


def html2text(html):
    """Use html2text but repair newlines cutting urls.
    Need to use this hack until
    https://github.com/aaronsw/html2text/issues/#issue/7 is not fixed"""
    txt = html2text_orig(html)
    links = list(LINK_RE.finditer(txt))
    out = StringIO()
    pos = 0
    for l in links:
        out.write(txt[pos:l.start()])
        out.write(l.group().replace('\n', ''))
        pos = l.end()
    out.write(txt[pos:])
    return out.getvalue()


class Mailer(object):
    """Mailer for generating and sending email
    In test mode the mailer always send mails but do not log it"""
    def __init__(self, email, test=False, verbose=0):
        self.test = test
        self.verbose = verbose
        self.email = email
        self.email_template = Template(self.email.content)
        self.title_template = Template(self.email.subject)


    @transaction.commit_manually
    def send(self):
        """Send the mail"""
        if not self.can_send:
            return

        try:
            self.smtp_connect()

            if self.verbose:
                print '%s email will be sent' % self.email.email_address

            self.attachments = self.build_attachments()

        # creo status messaggio
            if self.test:
                # imposto stati di test
                to_send_status = MailingStatus.TO_SEND_IN_TEST
                sent_status = MailingStatus.SENT_TEST
            else:
                # imposto stati di invio
                to_send_status = MailingStatus.TO_SEND
                sent_status = MailingStatus.SENT
            try:
                mail_status = MailingStatus.objects.create(
                    email_object=self.email,
                    status=to_send_status
                )
                self.email.set_status(mail_status.status)
                transaction.commit()
                if self.verbose == 2:
                    print "Email to send"
            except Exception, e:
                transaction.rollback()
                if self.verbose == 2:
                    print "Error Email to send:", e
                return

            message = self.build_message()

            try:
                from_addr=self.email.bounce_server.user
                to_addr=self.email.email_address
                msg=message.as_string()

                self.smtp.sendmail(
                    from_addr,
                    to_addr,
                    msg
                )
                mail_status = MailingStatus.objects.create(
                    email_object=self.email,
                    status=sent_status
                )
                self.email.set_status(mail_status.status)
                transaction.commit()
                if self.verbose == 2:
                    print "Email sent"
            except SMTPRecipientsRefused, e:
                mail_status = MailingStatus.objects.create(
                    email_object=self.email,
                    status=MailingStatus.INVALID
                )
                transaction.commit()
                if self.verbose == 2:
                    print "SMTP Recipient Refused:", e
            except Exception, e:
                mail_status = MailingStatus.objects.create(
                    email_object=self.email,
                    diagnostic_code = e,
                    status=sent_status
                )
                self.email.set_status(mail_status.status)
                transaction.commit()
                if self.verbose == 2:
                    print "SMTP error:", e
        except Exception, e:
            transaction.rollback()
            if self.verbose == 2:
                print "General error:", e
            return
        self.smtp.quit()

    def build_message(self):
        """
        Build the email as a multipart message containing
        a multipart alternative for text (plain, HTML) plus
        all the attached files.
        """
        content_html = self.build_email_content()
        content_text = html2text(content_html)

        message = MIMEMultipart()
        message['Subject'] = self.build_title_content()
        message['From'] = smart_str(self.email.header_sender)
        message['Reply-to'] = smart_str(self.email.header_reply)
        message['To'] = self.email.email_address
        message['List-ID'] = "<%s>" % self.email.id
        message_alt = MIMEMultipart('alternative')
        message_alt.attach(MIMEText(smart_str(content_text), 'plain', 'UTF-8'))
        message_alt.attach(MIMEText(smart_str(content_html), 'html', 'UTF-8'))
        message.attach(message_alt)

        for attachment in self.attachments:
            message.attach(attachment)

        for header, value in self.email.smtp_server.custom_headers.items():
            message[header] = value
        return message

    def build_attachments(self):
        """Build email's attachment messages"""
        attachments = []

        for attachment in self.email.attachments.all():
            ctype, encoding = mimetypes.guess_type(attachment.file_attachment.path)

            if ctype is None or encoding is not None:
                ctype = 'application/octet-stream'

            maintype, subtype = ctype.split('/', 1)

            fd = open(attachment.file_attachment.path, 'rb')
            if maintype == 'text':
                message_attachment = MIMEText(fd.read(), _subtype=subtype)
            elif maintype == 'message':
                message_attachment = message_from_file(fd)
            elif maintype == 'image':
                message_attachment = MIMEImage(fd.read(), _subtype=subtype)
            elif maintype == 'audio':
                message_attachment = MIMEAudio(fd.read(), _subtype=subtype)
            else:
                message_attachment = MIMEBase(maintype, subtype)
                message_attachment.set_payload(fd.read())
                encode_base64(message_attachment)
            fd.close()

            message_attachment.add_header(
                'Content-Disposition', 'attachment',
                filename=attachment.title
            )
            attachments.append(message_attachment)

        return attachments

    def smtp_connect(self):
        """Make a connection to the SMTP"""
        self.smtp = self.email.smtp_server.connect()

    def build_title_content(self):
        """Generate the email title"""
        context = Context({
            'email': self.email,
            'UNIQUE_KEY': ''.join(sample(
                UNIQUE_KEY_CHAR_SET, UNIQUE_KEY_LENGTH
            ))
        })
        title = self.title_template.render(context)
        return title

    def build_email_content(self):
        """Generate the mail"""
        uidb36, token = tokenize(self.email)
        context = Context({
            'email': self.email,
            'domain': Site.objects.get_current().domain,
            'uidb36': uidb36, 'token': token
        })

        content = self.email_template.render(context)
        # to track link
        if TRACKING_LINKS:
            content = track_links(content, context)

        if TRACKING_LINK_SITE:
            # to insert link to view email on site
            link_site = render_to_string(
                'mailer/email_link_site.html', context
            )
            content = body_insertion(content, link_site)

        # to insert image
        if TRACKING_IMAGE:
            image_tracking = render_to_string(
                'mailer/email_image_tracking.html', context
            )
            content = body_insertion(content, image_tracking, end=True)
        return smart_unicode(content)

    @property
    def can_send(self):
        """Check if the email can be sent"""
        if self.email.smtp_server.get_credits() <= 0:
            return False

        if self.test:
            return True

        if self.email.status == EmailObject.TO_SEND:
            return True

        return False