# -*- coding: utf-8 -*-

from HTMLParser import HTMLParseError

from django.contrib import admin, messages
from django.utils.translation import ugettext_lazy as _
from django.db.models import Q
from django.conf.urls.defaults import patterns, url

from jumbo.admin.options import ExtendibleModelAdmin, SettingsAdmin

from newsletter.models import Newsletter, MailingList, Settings, Contact
from newsletter.sender import Mailer

admin.site.register(Settings, SettingsAdmin)

##############################
# CONTACT                    #
##############################
class ContactAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("newsletter/css/admin_newsletter.css",)}

    list_display = (
        'mailinglist', 'content_object', 'date_subscription', 'date_unsubscription', 'is_test_contact',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    fieldsets = (
        (None, {'fields': (
            'mailinglist',
            ('date_subscription', 'date_unsubscription'),
            'email',
            'is_test_contact'
        )}),
    )
    actions = ['set_as_test']

    def get_actions(self, request):
        actions = super(ContactAdmin, self).get_actions(request)
        del actions['set_active']
        del actions['set_disactive']
        return actions

    def set_as_test(self, request, queryset):
        """
        Set contact as test
        """
        for contact in queryset:
            try:
                contact.test = True
                contact.save()
            except Exception, e:
                messages.error(request, _('Error: %s - %s' % contact, e))
    set_as_test.short_description =  _("Set Contacts as test")

##############################
# NEWSLETTER                 #
##############################
class NewsletterAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("newsletter/css/admin_newsletter.css",)}

    date_hierarchy = 'date_create'
    list_display = (
        'status', 'title', 'mailing_list', 'sending_date',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    fieldsets = (
        (None, {'fields': ('title', 'content',)}),
        (_('Receivers'), {'fields': ('mailing_list',)}),
        (_('Sending'), {'fields': (
            'sending_date', 'status', 'slug'
        )}),
    )
    prepopulated_fields = {'slug': ('title',)}
    actions = ['send_newsletter_test', 'make_ready_to_send', 'make_cancel_sending']

    def get_actions(self, request):
        actions = super(NewsletterAdmin, self).get_actions(request)
        del actions['set_active']
        del actions['set_disactive']
        if not request.user.has_perm('newsletter.can_change_status'):
            del actions['make_ready_to_send']
            del actions['make_cancel_sending']
        return actions

    def send_newsletter_test(self, request, queryset):
        """Send newsletter in test"""
        for newsletter in queryset:
            if newsletter.mailing_list.has_test_contact:
                mailer = Mailer(newsletter, test=True)
                try:
                    mailer.run()
                except HTMLParseError:
                    self.message_user(request, _('Unable send newsletter, due to errors within HTML.'))
                    continue
                self.message_user(request, _('%s succesfully sent.') % newsletter)
            else:
                self.message_user(request, _('No test contacts assigned for %s.') % newsletter)
    send_newsletter_test.short_description = _('Send test newsletter')

    def make_ready_to_send(self, request, queryset):
        """Make newsletter ready to send"""
        queryset = queryset.filter(status=Newsletter.DRAFT)
        for newsletter in queryset:
            newsletter.status = Newsletter.WAITING
            newsletter.save()
        self.message_user(request, _('%s newletters are ready to send') % queryset.count())
    make_ready_to_send.short_description = _('Make ready to send')

    def make_cancel_sending(self, request, queryset):
        """Cancel the sending of newsletters"""
        queryset = queryset.filter(Q(status=Newsletter.WAITING) |
                                   Q(status=Newsletter.SENDING))
        for newsletter in queryset:
            newsletter.status = Newsletter.CANCELED
            newsletter.save()
        self.message_user(request, _('%s newletters are cancelled') % queryset.count())
    make_cancel_sending.short_description = _('Cancel the sending')

##############################
# MAILING LIST               #
##############################
class MailingListAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("newsletter/css/admin_newsletter.css",)}

    date_hierarchy = 'date_create'
    list_display = (
        'date_create', 'name', 'description',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    list_editable = ('name', 'description')
    fieldsets = (
        (None, {'fields': ('name', 'description',)}),
    )
#    actions = ['merge_mailinglist']


#    def merge_mailinglist(self, request, queryset):
#        """Merge multiple mailing list"""
#        if queryset.count() == 1:
#            self.message_user(request, _('Please select a least 2 mailing list.'))
#            return None
#
#        subscribers = {}
#        unsubscribers = {}
#        for ml in queryset:
#            for contact in ml.subscribers.all():
#                subscribers[contact] = ''
#            for contact in ml.unsubscribers.all():
#                unsubscribers[contact] = ''
#
#        when = str(datetime.now()).split('.')[0]
#        new_mailing = MailingList(name=_('Merging list at %s') % when,
#                                  description=_('Mailing list created by merging at %s') % when)
#        new_mailing.save()
#        new_mailing.subscribers = subscribers.keys()
#        new_mailing.unsubscribers = unsubscribers.keys()
#
#        if not request.user.is_superuser and USE_WORKGROUPS:
#            for workgroup in request_workgroups(request):
#                workgroup.mailinglists.add(new_mailing)
#
#        self.message_user(request, _('%s succesfully created by merging.') % new_mailing)
#        return HttpResponseRedirect(reverse('admin:newsletter_mailinglist_change',
#                                            args=[new_mailing.pk]))
#    merge_mailinglist.short_description = _('Merge selected mailinglists')

#    def exportation_link(self, mailinglist):
#        ## FIX vcard.py - manca first_name e last_name
#        """Display link for exportation"""
#        return u'<a href="%s">%s</a>' % (
#            reverse(
#                'admin:newsletter_mailinglist_export',
#                args=[mailinglist.pk]
#            ),
#            _('Export Subscribers')
#        )
#    exportation_link.allow_tags = True
#    exportation_link.short_description = _('Export')

#    def export_subscribers(self, request, mailinglist_id):
#        """Export subscribers in the mailing in VCard"""
#        mailinglist = get_object_or_404(MailingList, pk=mailinglist_id)
#        name = 'contacts_%s' % mailinglist.name
#        return vcard_contacts_export_response(mailinglist.subscribers.all(), name)

    ### FIX vcard.py - manca first_name e last_name
    #def get_urls(self):
    #    urls = super(MailingListAdmin, self).get_urls()
    #    my_urls = patterns('',
    #        url(
    #            r'^export/(?P<mailinglist_id>\d+)/$',
    #            self.admin_site.admin_view(self.export_subscribers),
    #            name='newsletter_mailinglist_export'
    #        ),
    #    )
    #    return my_urls + urls

admin.site.register(Contact, ContactAdmin)
admin.site.register(Newsletter, NewsletterAdmin)
admin.site.register(MailingList, MailingListAdmin)