"""Models for newsletter"""
from smtplib import SMTP
from smtplib import SMTPHeloError
from datetime import datetime, time
from datetime import timedelta
from picklefield.fields import PickledObjectField

from django.db import models
from django.db.models import Q
from django.utils.encoding import smart_str
from django.db import connection, transaction
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import Group
from django.utils.encoding import force_unicode


from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel, HTMLField
from jumbo.db.models import Settings as JSettings
from jumbo.utils.functions import get_settings

class MailingList(UserModel, DateModel):
    """Mailing list"""
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255
    )
    description = models.TextField(
        verbose_name=_('description'),
        blank=True, null=True
    )

    def get_subscribers(self):
        """ return Subscribers Contact queryset """
        return self.contacts.filter(
            date_unsubscription__isnull=True
        )
    subscribers = property(get_subscribers)

    def get_test_subscribers(self):
        """ return Test Subscribers Contact queryset """
        return self.subscribers.filter(
            is_test_contact=True
        )
    test_subscribers = property(get_test_subscribers)

    def get_unsubscribers(self):
        """ return Unsubscribers Contact queryset """
        return self.contacts.filter(
            date_unsubscription__isnull=False,
        )
    unsubscribers = property(get_unsubscribers)

    def has_test_contact(self):
        """ return True or False if Test Contacts are present """
        if self.test_subscribers.count() > 0:
            return True
        else:
            return False
    has_test_contact = property(has_test_contact)

    def add_subscriber(self, object):
        try:
            contact = Contact.objects.get(
                mailinglist=self, email=object.email
            )
            return (False, _("Email already exists"))

        except:
            contact = Contact(
                mailinglist=self,
                content_object=object,
                email=object.email
            )
            contact.save()
            return (True, contact)

    def clone(self):
        today = datetime.today()
        new_obj = MailingList()
        new_obj.name = '%s_%s' % (self.name, today.strftime('%d-%m-%Y'))
        new_obj.description = self.description
        new_obj.save()

        return new_obj

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ('-date_create',)
        verbose_name = _('mailing list')
        verbose_name_plural = _('mailing lists')

class Contact(UserModel, DateModel):
    """
    Email contact
    """
    mailinglist = models.ForeignKey(
        MailingList,
        verbose_name=_("mailing list"),
        related_name='contacts'
    )
    # generic object with email field
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    email =models.EmailField(
        verbose_name="email",
    )
    date_subscription = models.DateTimeField(
        verbose_name="date subscription",
        default=datetime.today
    )
    date_unsubscription = models.DateTimeField(
        verbose_name="date unsubscription",
        blank=True, null=True
    )
    is_test_contact = models.BooleanField(
        verbose_name=_('test'),
    )

    class Meta:
        ordering = ('email',)
        verbose_name = _('Contact')
        verbose_name_plural = _('Contacts')
        unique_together = ('mailinglist', 'email')

    def __unicode__(self):
        return "%s: %s" % (
            self.mailinglist.name,
            self.email
        )


class Newsletter(UserModel, DateModel):
    """Newsletter to be sended to contacts"""
    DRAFT = 0
    WAITING = 1
    SENDING = 2
    SENT = 4
    CANCELED = 5
    STATUS_CHOICES = (
        (DRAFT, _('draft')),
        (WAITING, _('waiting sending')),
        (SENDING, _('sending')),
        (SENT, _('sent')),
        (CANCELED, _('canceled')),
    )

    status = models.IntegerField(
        _('status'),
        choices=STATUS_CHOICES,
        default=DRAFT
    )
    title = models.CharField(
        verbose_name=_('title'),
        max_length=255,
        help_text=_("""
            You can use the "{{ UNIQUE_KEY }}" variable
            for unique identifier within the newsletter's title.
        """)
    )
    slug = models.SlugField(
        help_text=_('Used for displaying the newsletter on the site.'),
        unique=True
    )
    content = HTMLField(
        _('content'),
        help_text=_('write HTML code or paste an URL.'),
    )
    mailing_list = models.ForeignKey(
        MailingList,
        related_name = 'newsletters',
        verbose_name=_('mailing list')
    )
    sending_date = models.DateTimeField(
        _('sending date'),
        default=datetime.now
    )

    def clone(self):
        today = datetime.today()
        new_obj = Newsletter()
        new_obj.title = '%s_%s' % (self.title, today.strftime('%d-%m-%Y'))
        new_obj.slug = new_obj.title
        new_obj.content = self.content
        new_obj.mailing_list = self.mailing_list
        new_obj.server = self.server
        new_obj.header_sender = self.header_sender
        new_obj.header_reply= self.header_reply
        new_obj.save()

        return new_obj

    def subscriptions(self):
        return self.mailing_list.subscribers.count()
    subscriptions = property(subscriptions)

    def unsubscriptions(self):
        return self.mailing_status.unsubscribers.count()
    unsubscriptions = property(unsubscriptions)

    def unsubscriptions_perc(self):
        if self.subscriptions != 0:
            return float(self.unsubscriptions) / float(self.subscriptions) * 100
        return float(0)
    unsubscriptions_perc = property(unsubscriptions_perc)

    @models.permalink
    def get_absolute_url(self):
        return ('newsletter_newsletter_preview', (self.slug,))

    @models.permalink
    def get_historic_url(self):
        return ('newsletter_newsletter_historic', (self.slug,))

    @models.permalink
    def get_statistics_url(self):
        return ('newsletter_newsletter_statistics', (self.slug,))

    def __unicode__(self):
        return self.title

    class Meta:
        ordering = ('-date_create',)
        verbose_name = _('newsletter')
        verbose_name_plural = _('newsletters')
        permissions = (
            ('can_change_status', 'Can change status'),
            ('view_newsletter',   'Can view newsletter'),
        )

class Settings(JSettings):
    pass